'use strict';

import request from 'request';
import extend from 'extend';

class CamperApi {

    constructor(settings, logger) {

        this.baseUrl = 'https://booking.camperdays.de/api/v1';

        if (typeof settings !== 'object') {
            return false;
        }

        if (!(this instanceof CamperApi)) {
            return new CamperApi(token);
        }

        this.logger = (typeof logger === 'undefined') ? console : logger;

        this.settings = extend(true, {
            logErrors: false,
        }, settings);

        this.accessToken = settings.accessToken;
    }

    _createRequest(options, qs) {

        return {
            method: options.method,
            url: this.baseUrl + options.path,
            qs: qs,
            headers: {
                'Authorization': 'Basic ' + this.accessToken,
                'Accept': 'application/json'
            },
            timeout: 1500,
            body: options.body,
            json: true
        };

    }

    _executeRequest(options, querystring, callback) {

        if (!callback) {
            callback = querystring;
            querystring = undefined;
        }

        const req = this._createRequest(options, querystring);

        request(req, (err, response, body) => {

            if (this.settings.logErrors) {
                this.log(req.url, err);
            }

            if (!body) {
                return callback(null, null);
            }

            if (!err) {
                return callback(body.error, body.result || null);
            }

            callback(err, null);
        });

    }

    /**
     * Log Errors.
     *
     * GET /destination
     * @param name STRING
     * @param err {errorObject}
    */
    log(name, err) {

        if (!err) {
            return null;
        }

        if (err && err.code === 'ETIMEDOUT') {

            if (err.connect === true) {
                return this.logger.error('ApiCall | %s | Request connection timeout', name);
            }

            this.logger.error('ApiCall | %s | Request timeout', name);
        }
    }

    /**
     * Returns a destination object.
     *
     * GET /destination
     * @param callback {requestCallback} The callback that will manage the response.
    */
    destination(callback) {

        this._executeRequest({
            method: 'GET',
            path: '/destination',
        }, callback);

    }

    /**
     * Returns a search object.
     *
     * POST /search
     * @param searchData {object} search request data
     * @param callback {requestCallback} The callback that will manage the response.
    */
    search(searchData, callback) {

        this._executeRequest({
            method: 'POST',
            body: searchData,
            path: '/search',
        }, callback);

    }

    /**
     * Returns a result object.
     *
     * GET /search/:requestId/offer
     * @param requestId {string} result list identifier
     * @param callback {requestCallback} The callback that will manage the response.
    */
    result(requestId, callback) {

        this._executeRequest({
            method: 'GET',
            path: '/search/' + requestId + '/offer',
        }, callback);

    }

    /**
     * Returns a offer object.
     *
     * GET /search/:requestId/offer
     * @param requestId {string} restrict the set of offers
     * @param offerId {string} offer identifier
     * @param callback {requestCallback} The callback that will manage the response.
    */
    offer(requestId, offerId, callback) {

        this._executeRequest({
            method: 'GET',
            path: '/search/' + requestId + '/offer/' + offerId,
        }, callback);

    }

    /**
     * Returns a booking object.
     *
     * POST /booking
     * @param bookingData {object} bookingData
     * @param callback {requestCallback} The callback that will manage the response.
    */
    booking(bookingData, callback) {

        this._executeRequest({
            method: 'POST',
            body: bookingData,
            path: '/booking',
        }, callback);

    }

}

module.exports = CamperApi;
